#include "main.h"
#include "sequence.h"
#include "single.h" 

MyApp App;

void WaitForKey()
{
	cout << "\nPlease press [RETURN] to continue: " << flush;
	cin.clear();
	cin.ignore(cin.rdbuf()->in_avail() , '\n');

	cin.get();
	cout << endl;
}


bool MyApp::Init(int argc, char **argv)
{

	printf("SmartCropper V2.5 by Seth A. Robinson\n");

	argp.add_usage("[Input file left-based substring] [Optional addition parms]");

	argp.add_doc("Crops a sequence of images to the combined smallest enclosing rect.");
	argp.add_doc("\nInput with alpha information should be used, such as .tga or .png.  Output will always be .png.");
	argp.add_doc("\nExample 1:  smartcropper.exe Birds* -o CroppedBirds -p 2");
	argp.add_doc("\nExample 2:  smartcropper.exe image_with_pink_colorkey_five_frames.bmp -x 5 -t 255,0,255");

	argp.add_group ("Options:");
	argp.add_option('p', "pad", "number", "Clear pixels to be added around the border as padding");
	argp.add_option('o', "output", "file", "Sequence base name to write to, if not specified, originals will be overwritten!");
	argp.add_option('h', "help", "", "This help text");
	argp.add_option('x', "framex", "number", "Horizontal frames per image.  If specified, multiple images are processed within an image.");
	argp.add_option('y', "framey", "number", "Vertical frames per image.  If specified, multiple images are processed within an image.");
	argp.add_option('t', "transparentcolor", "file", "Use any RGB color for transparency, when alpha isn't available.  example: t 255,0,255");
	
	argp.parse_args(argc, argv);

	while (argp.next())
	{
		switch (argp.get_key()) 
		{
		case 'f':
			std::cout << "file: " << argp.get_argument() << std::endl;
			break;
		case 'o':
			m_fileOutput = argp.get_argument();
			break;
		
		case 'p':
			m_padding = CL_String::to_int(argp.get_argument());
			break;
		case 't':
			{
				string temp = argp.get_argument();
				
				int r = CL_String::to_int(temp);
				//cut off the first # we just took
				temp = temp.substr(temp.find_first_of(',')+1, temp.size());
				int g = CL_String::to_int(temp);
				temp = temp.substr(temp.find_first_of(',')+1, temp.size());
				int b = CL_String::to_int(temp);
				m_colorKey = CL_Color(r,g,b);

				m_bUsesColorKey = true;
			}
			break;

		case 'x':
			m_framesX = CL_String::to_int(argp.get_argument());
			break;
		case 'y':
			m_framesY = CL_String::to_int(argp.get_argument());
			break;

		case CL_CommandLine::REST_ARG:
			m_fileInput = argp.get_argument();
			break;
		default:
			std::cout << "Got " << argp.get_key() << " " << argp.get_argument() << std::endl;
			break;
		}
	}

	if (m_fileInput.empty())
	{
		argp.print_help();
		return false;
	}
	if (m_fileOutput.empty())
	{
		m_fileOutput = m_fileInput;
	}
	return true; //no error
}

void log_to_cout(const std::string &channel, int level, const std::string &message)
{
	cout << message;
#ifdef WIN32
	OutputDebugString(message.c_str()); //help me debug easier
#endif
}

int main(int argc, char **argv)
{
	// Initialize ClanLib base components manually, because it's a console app
	CL_SetupCore::set_instance(GetModuleHandle(NULL));
	CL_SetupCore::init(true);

	//this is so we can trap clanlib log messages too
	CL_Slot slot_log = CL_Log::sig_log().connect(&log_to_cout);

	// Initialize the ClanLib display component
	CL_SetupDisplay setup_display;

	try
		{
			if (App.Init(argc, argv))
			{
				//ok, continue
			
				if (App.m_framesX > 1 || App.m_framesY > 1)
				{
					//special handling
					Single singleImage(App.m_fileInput, App.m_fileOutput);
					singleImage.Process(); //actually do it

				} else
				{
					Sequence seq(App.m_fileInput, App.m_fileOutput);
					seq.Process(); //actually do it
				}
				
			}
		
		}
		catch(CL_Error error)
		{
			std::cout << "Exception caught : " << error.message.c_str() << std::endl;			
			OutputDebugString(error.message.c_str());
			WaitForKey();
		}

		CL_SetupCore::deinit();
	
#ifdef _DEBUG
		WaitForKey();
#endif
		return 0; //ok
}




//misc utils


//snippet from Zahlman's post on gamedev:  http://www.gamedev.net/community/forums/topic.asp?topic_id=372125
void StringReplace(const std::string& what, const std::string& with, std::string& in)
{
	size_t pos = 0;
	size_t whatLen = what.length();
	size_t withLen = with.length();
	while ((pos = in.find(what, pos)) != std::string::npos)
	{
		in.replace(pos, whatLen, with);
		pos += withLen;
	}
}

CL_Rect GetBoundsOfCombinedRects(const CL_Rect &a, const CL_Rect &b)
{
	CL_Rect r;

	r.left = min(a.left, b.left);
	r.right = max(a.right, b.right);
	r.top = min(a.top, b.top);
	r.bottom = max(a.bottom, b.bottom);

	return r;
}


void ClearPixelBuffer(CL_PixelBuffer* pPixelBuffer, CL_Color color)
{
	cl_assert(pPixelBuffer && "Invalid buffer!");
	int bytes_pp = pPixelBuffer->get_format().get_depth()/8;
	cl_assert(bytes_pp == 4 && "We only support 32 bit 8888 format right now.");

	unsigned int dest_format_color = color.to_pixelformat(pPixelBuffer->get_format());
	int copy_count = pPixelBuffer->get_pitch()/4;

	unsigned int *p_data = (unsigned int*)pPixelBuffer->get_data();

	pPixelBuffer->lock();
	for (int y=0; y < pPixelBuffer->get_height(); y++)
	{
		for (int x=0; x < copy_count; x++)
		{
			*p_data = dest_format_color;
			p_data++;
		}
	}
	pPixelBuffer->unlock();
}



//behind the scenes CL is doing reference counting on the buffer, so don't worry, this won't copy the whole image

bool IsNotTransparent(CL_Color pixel)
{
	if (pixel.get_alpha() == 0) return false;
	if (App.m_bUsesColorKey)
	{
		if (pixel == App.m_colorKey) return false;
	}

	return true; //not transparnet here
}

CL_Rect GetBoundsFromImage(CL_PixelBuffer buf)
{
	
	CL_Rect r = CL_Rect(0,0, buf.get_width(), buf.get_height());

	//scan for left!

	bool bFound = false;

	//scan left
	for (; r.left < r.right; r.left++)
	{
		for (int y=r.top; y < r.bottom; y++)
		{
			if (IsNotTransparent(buf.get_pixel(r.left, y)))
			{
				//the pixel is NOT transparent
				bFound = true;
				goto done1;
			}
		}
	}
done1: //yes, I suck

	//scan from right
	for (; r.right > r.left; r.right--)
	{
		for (int y=r.top; y < r.bottom; y++)
		{
			if (IsNotTransparent(buf.get_pixel(r.right-1, y)))
			{
				//the pixel is NOT transparent
				goto done2;
			}
		}
	}

done2: //yes, I suck

	//scan from top
	for (; r.top < r.bottom; r.top++)
	{
		for (int x=r.left; x < r.right; x++)
		{
			if (IsNotTransparent(buf.get_pixel(x, r.top)))
			{
				//the pixel is NOT transparent
				goto done3;
			}
		}
	}

done3: //yes, I suck

	//scan from bottom
	for (; r.bottom > r.top; r.bottom--)
	{
		for (int x=r.left; x < r.right; x++)
		{
			if (IsNotTransparent(buf.get_pixel(x, r.bottom-1)))
			{
				//the pixel is NOT transparent
				goto done4;
				break;
			}
		}
	}

done4: //yes, I suck

	if (!bFound)
	{
		//blank image
		//r.left = -1; //it will detect this and know what's up
		 r = CL_Rect(buf.get_width()/2,buf.get_height()/2, buf.get_width()/2, buf.get_height()/2);
	}
	
	return r;
}

string GetValidOutputType(string fileName)
{
	if (CL_String::get_extension(fileName) == "bmp")
	{
		printf("Cannot save bitmap yet.  Doing png instead.");
		fileName = fileName.substr(0, fileName.size()-3) + "png";
	}

	return fileName;
}


