#include "sequence.h"

Sequence::Sequence(string input, string output): m_input(input), m_output(output)
{
	//figure out relative path.  CL_String::get_path doesn't do a good job with relative paths.. do it manually.
	
	StringReplace("\\", "/", m_input);
	size_t index = m_input.find_last_of("/");
	if (index != -1)
	{
		//found a path section
		m_inputDir = m_input.substr(0, index+1);
	}

	//same thing for figuring out the output dir
	StringReplace("\\", "/", m_output);
	index = m_output.find_last_of("/");
	if (index != -1)
	{
		//found a path section
		m_outputDir = m_output.substr(0, index+1);
	}


}

void Sequence::BuildInputFileList()
{
	CL_DirectoryScanner scanner;
	scanner.scan(m_inputDir, CL_String::get_filename(m_input)+"*");

	while (scanner.next())
	{
		std::string file = scanner.get_name();
		if (!scanner.is_directory())
		{
			m_fileArray.push_back(scanner.get_name());
			//CL_Log::log("Sequence", "Adding %1\n", scanner.get_name());
		}
	}

	if (m_fileArray.size() < 1)
	{
		throw CL_Error("Can't find any input files using the dir and search substring of " + m_input);
	}

	CL_Log::log("Sequence", "Found %1 images to process.\n", int(m_fileArray.size()));

}

void Sequence::FindSmallestEnclosingRect()
{

	bool bFirst = true;

	for (unsigned int i=0; i < m_fileArray.size(); i++)
	{
		CL_Log::log("", "%1 - ", m_inputDir+m_fileArray[i]);

		CL_PixelBuffer pixBuff = CL_ProviderFactory::load(m_inputDir+m_fileArray[i]);
		CL_Rect activeRect = GetBoundsFromImage(pixBuff);

		CL_Log::log("", "Rect [%1, %2, %3, %4] (Size: %5 X %6)\n", activeRect.left, activeRect.top, activeRect.right, activeRect.bottom,
			activeRect.get_width(), activeRect.get_height());

		if (activeRect.left == -1)
		{
			//signal a blank image, don't use this data..
			CL_Log::log("", "** Warning: Blank image detected.  Ignoring for size calculations. **\n");
			continue;
		}

		if (bFirst)
		{
			m_originalRect = CL_Rect(0,0, pixBuff.get_width(), pixBuff.get_height());
			m_enclosingRect = activeRect;
			bFirst = false;
		} else
		{
			if (m_originalRect != CL_Rect(0,0, pixBuff.get_width(), pixBuff.get_height()))
			{
				throw CL_Error("Error, all images need to be the exact same dimensions for this to work!");
			}

			//enlarge enclosing rect to fit the new rect if needed
			m_enclosingRect = GetBoundsOfCombinedRects(m_enclosingRect, activeRect);
		}
	}

	CL_Log::log("", "Final bounding Rect [%1, %2, %3, %4] (Size: %5 X %6)\n", m_enclosingRect.left, m_enclosingRect.top, m_enclosingRect.right, m_enclosingRect.bottom,
		m_enclosingRect.get_width(), m_enclosingRect.get_height());
}

void Sequence::WriteCroppedImages()
{
	for (unsigned int i=0; i < m_fileArray.size(); i++)
	{
		//load the image yet again (we could easily cache these but.. if you had a LOT of images might become a memory problem...)
		CL_PixelBuffer pixBuff = CL_ProviderFactory::load(m_inputDir+m_fileArray[i]);
	
		//apply padding
		CL_Rect paddedRect = m_enclosingRect;
		
		if (App.m_padding > 0) 
		{
			//they want a border..
			paddedRect.right += App.m_padding;
			paddedRect.left -= App.m_padding;
			paddedRect.top -= App.m_padding;
			paddedRect.bottom += App.m_padding;
		}
		//crop it

		CL_PixelBuffer finalBuff(paddedRect.get_width(), paddedRect.get_height(), paddedRect.get_width()*4, CL_PixelFormat::abgr8888);

		if (App.m_padding > 0) 
		{
			//if padding was used, we better fill the rect first so it's clear
			ClearPixelBuffer(&finalBuff, CL_Color(0,0,0,0)); //a transparent pixel
		}

		//copy the cropped position into the new buffer
		CL_Rect destRect = CL_Rect(App.m_padding,App.m_padding, finalBuff.get_width()-App.m_padding, finalBuff.get_height()-App.m_padding);
		pixBuff.convert(finalBuff.get_data(), finalBuff.get_format(), finalBuff.get_pitch(), destRect, m_enclosingRect);

		//save it
		string postfix = m_fileArray[i].substr(CL_String::get_filename(m_input).length(), m_fileArray[i].length());

		//hmm, we better force it to be a .png extension, CL doesn't handle much right now for saving
		postfix = postfix.substr(0, postfix.length()-4)+".png";

		string finalOutputImage = m_outputDir+CL_String::get_filename(m_output)+postfix;
		CL_Log::log("", "Writing %1...\n", finalOutputImage);

		//write the final version of this image
		CL_ProviderFactory::save(finalBuff, finalOutputImage);
	}
}


bool Sequence::Process()
{
	BuildInputFileList();
	FindSmallestEnclosingRect();
	WriteCroppedImages();
	return true; //no erro
}